create or replace
PACKAGE BODY 
---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2009. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/dictionary/wm_scenarioupdate_pkgb.sql BL_core_schema_8.0.7_8.0.7.2.02_20091208  043145
-- (#) 
--
WM_SCENARIOUPDATE IS

   /*
	****************************************************************************************
	PACKAGE:    SCENARIO_UPDATE
	DATE :		18 MAY 2009

	DESCRIPTION:	This package is designed to handle all changes to the XML strings that
				    make up the Scenarios for the Classic Client. The scenarios are stored in
				    the following tables:
					    PM_SCENARIO, WMQ_QUERY, WMQ_QUERY_TEXT

				    There is one row per entity in the WMQ_QUERY_TEXT table regardless of
				    whether that entity has intances in the scenario or not. The row contains the XML string
				    which specifies which instances of that entity are included in the Scenario.

                    All the XML strings are in the following format:

                         <CLASS Name='MSC'>
                         <FILTER><![CDATA[inGroup(Id,"1963") OR (!inGroup(ParentId,"1964") AND Active)]]></FILTER>
                         <ID/>
                         <PARENT/>
                         <SORT Order='Ascending'><![CDATA[stringToInt(LocalId)]]></SORT>
                         <DATAMEMBER Name='DefaultName'><![CDATA[EntityType.Name + ":" + LocalId]]></DATAMEMBER> <DATAMEMBER Name='Name'/>
                         <DATAMEMBER Name='Label'><![CDATA[TheLabel]]></DATAMEMBER>
                         <DATAMEMBER Name='Order'><![CDATA[stringToInt(LocalId)]]></DATAMEMBER>
                         <DATAMEMBER Name='Active'/> <DATAMEMBER Name='Placeholder'/>
                         <DATAMEMBER Name='Unlinked'><![CDATA[inGroup(Id,"1963") OR !Active]]></DATAMEMBER>
                         <DATAMEMBER Name='Excluded'><![CDATA[0]]></DATAMEMBER>
                         </CLASS>

	*****************************************************************************************
	*/

  gContextId	wmn_context.context_id%type;

   /*
    PROCEDURE:    addInstance
    DESCRIPTION:  This procedure handles adding a new instance to a User Scenario
    PARAMS:			pInstanceKey - the full key of the instance to be added
        			pEntity      - the entity name of the instance
        			pScenario    - the name of the target Scenario to add to
                    retCode		 - the return code
    */
  PROCEDURE addinstance(pinstancekey IN wmn_instance_context.fullkey%TYPE,
                             pentityname  IN wmn_entity.entity_name%TYPE,
                             pscenario    IN pm_scenario.name%TYPE,
                             pFolderPath	IN VARCHAR2,
                             retcode OUT NUMBER)
  IS

  invalid_context EXCEPTION;
  invalid_entity EXCEPTION;
  invalid_fullkey EXCEPTION;
  invalid_scenario EXCEPTION;
  invalid_folder EXCEPTION;
  already_exists EXCEPTION;

  ventityid wmn_entity.entity_id%TYPE;
  vinstanceid wmn_instance.instance_id%TYPE;
  vscenarioqueryid pm_scenario.query_id%TYPE;
  vcheckexistence boolean := FALSE;
  vFolderId	pm_folder.id%TYPE;

  BEGIN

   /*
   BEGIN
      rpt_file2  := utl_file.fopen('/tmp', 'addinstance.rep', 'W');
    EXCEPTION
       WHEN OTHERS THEN
          log_msg('Error opening file ' ||TO_CHAR(SQLCODE, '99999'));
          retcode := 1;
          RETURN;
    END;
  */

   retcode := 0;


    log_msg('*************************************');
    log_msg('Starting addInstance with the following params:  ');
    log_msg('instanceKey: ' || pinstancekey);
    log_msg('Entity: ' || pentityname);
    log_msg('pScenario: ' || pscenario);
    log_msg('*************************************');
    log_msg('');

    /* Get the correct context id */
    SELECT context_id into gContextId
    FROM wmn_context
    WHERE context_use = 'S';

    IF gContextId IS NULL THEN
    	raise invalid_context;
    END IF;


    /* Get the entity Id */
    ventityid := wm_network.get_entity(pentityname);

    IF ventityid IS NULL THEN
      raise invalid_entity;
    END IF;


    /* Get the instance Id */
    vinstanceid := validatefullkey(pinstancekey,   ventityid);

    IF vinstanceid IS NULL THEN
      raise invalid_fullkey;
    END IF;


    /* Get the folder id */
    vFolderId := getFolderIdByPath(pFolderpath);

    IF vFolderId IS NULL THEN
      raise invalid_folder;
    END IF;

    /* Get the scenario Id */
    vscenarioqueryid := validatescenario(pscenario, vFolderId);

    IF vscenarioqueryid IS NULL THEN
      raise invalid_scenario;
    END IF;

    /* verify if already exists */

    vcheckexistence := checkexistence(vinstanceid, ventityid, vscenarioqueryid);

    IF vcheckexistence THEN
       raise already_exists;
    END IF;


    /* Call the procedure to update which instances are included in the scenario */

    updatescenarioxml(vinstanceid,   ventityid,   vscenarioqueryid);

    walkUpentitytree(vinstanceid,    ventityid,   vscenarioqueryid);

    walkDownentitytree(vinstanceid,  vscenarioqueryid);

    COMMIT;

    utl_file.fclose(rpt_file2);

   -- raise_application_error(-20002,   'Entity ' || pentityname || ' is not valid');

    EXCEPTION
    WHEN invalid_context THEN
	log_msg('======================================================= ');
	log_msg('Unable to find Scenario context in WMN_CONTEXT');
	log_msg('======================================================= ');
	retcode := 1;
    WHEN invalid_entity THEN
	log_msg('======================================================= ');
	log_msg('Entity ' || pentityname || ' is not valid');
	log_msg('======================================================= ');
	retcode := 1;
    WHEN invalid_fullkey THEN
	log_msg('======================================================= ');
	log_msg('Instance key' || pinstancekey || ' is not valid');
	log_msg('======================================================= ');
	retcode := 1;
    WHEN invalid_scenario THEN
	log_msg('======================================================= ');
	log_msg('Scenario' || vscenarioqueryid || ' is not valid');
	log_msg('======================================================= ');
	retcode := 1;
    WHEN invalid_folder THEN
	log_msg('======================================================= ');
	log_msg('Unable to resolve folder path ' || pFolderPath );
	log_msg('======================================================= ');
	retcode := 1;
    WHEN already_exists THEN
	log_msg('======================================================= ');
	log_msg('The instance that the user is trying to add to the scenario already exists in that scenario');
	log_msg('======================================================= ');
	retcode := 1;
    WHEN OTHERS THEN
      log_msg('!Error. scenario update has failed... ' || SQLCODE || ' == ' || SQLERRM);
      ROLLBACK;
    END;

  /*
    FUNCTION:	  checkalready exists
    DESCRIPTION:  This function checks if instance already exists
    PARAMS:			vinstanceid - instance id associated with full key
    				pEntityId - the Entity Id associated with the full key
                                pScenarioId
	RETURNS:		The correct Instance Id
    */
  FUNCTION checkexistence(pInstanceid IN wmn_instance.instance_id%TYPE,
                           pEntityid    IN wmn_entity.entity_id%TYPE,
                           pScenarioqueryid IN pm_scenario.query_id%TYPE)
  RETURN BOOLEAN
  IS
   vXml_exists CLOB;
  BEGIN

    vXml_exists := getXML(pScenarioQueryId, pEntityId);

    IF instr(vXml_exists, pInstanceid) > 0 THEN
       RETURN TRUE;
    ELSE
       RETURN FALSE;
    END IF;

  EXCEPTION
    WHEN no_data_found THEN
    RETURN FALSE;
  END;

  /*
    FUNCTION:	  validateFullKey
    DESCRIPTION:  This function validates the InstanceKey
    PARAMS:			pInstanceKey - the full key of the instance to be added
    				pEntityId - the Entity Id associated with the full key
	RETURNS:		The correct Instance Id
    */
  FUNCTION validatefullkey(pinstancekey IN wmn_instance_context.fullkey%TYPE,
                           pentityid    IN wmn_entity.entity_id%TYPE)
  RETURN wmn_instance.instance_id%TYPE
  IS
   vinstanceid wmn_instance.instance_id%TYPE;
  BEGIN

    /* Check that an instance exists for this entity with this full key */
    SELECT instance_id
    INTO vinstanceid
    FROM wmn_instance_context
    WHERE entity_id = pentityid
     AND fullkey = pinstancekey
     AND context_id = gContextId;

    RETURN vinstanceid;

  EXCEPTION
  WHEN no_data_found THEN
    RETURN(NULL);
  END;

  /*
    FUNCTION:	  validateScenario
    DESCRIPTION:  This function validates the Scenario Name and returns the Scenario Id
    PARAMS:			pScenario - the name of the Scenario
	RETURNS:		The correct Scenario Query Id
    */
  FUNCTION validatescenario(pscenario IN pm_scenario.name%TYPE,
  							pFolderId	IN pm_folder.id%TYPE)
  RETURN pm_scenario.query_id%TYPE
  IS
     vscenarioqueryid 	pm_scenario.query_id%TYPE;

  BEGIN

    /* Get Scenario Query Id and return correct error codes if not found */
    SELECT query_id
    INTO vscenarioqueryid
    FROM pm_scenario
    WHERE name = pscenario
    AND type = 'U'
    AND FOLDERID = pFolderId;

    RETURN vscenarioqueryid;

  EXCEPTION
  WHEN no_data_found THEN
    RETURN(NULL);
  END validatescenario;


	/*
    FUNCTION:	  getFolderIdByPath
    DESCRIPTION:  This function takes in a folder path and returns the correct
    				folder id for it
    PARAMS:			pFolderPath - the full path of the folder
	RETURNS:		The correct Folder Id
    */

  FUNCTION getFolderIdByPath(pFolderPath VARCHAR2)
  RETURN NUMBER
  AS
    vFolder      	pm_folder.NAME%TYPE := 'ROOT';
    vFolderPath     VARCHAR2(4000) := rtrim(ltrim(REPLACE(pFolderPath, '\', '/'), '/'),'/') || '/';
    vFolderId       pm_folder.id%TYPE;
    vParentFolderId pm_folder.folderid%TYPE := NULL;

  BEGIN

    LOOP
      EXIT WHEN vFolder IS NULL;
      SELECT id
        INTO vFolderId
        FROM pm_folder
       WHERE upper(NAME) = upper(vFolder)
         AND decode(folderid, NULL, -1, folderid) = nvl(vParentFolderId, -1);
      vParentFolderId := vFolderId;
      vFolder      := substr(vFolderPath, 1, instr(vFolderPath, '/') - 1);
      vFolderPath      := substr(vFolderPath,
                             instr(vFolderPath, '/') + 1,
                             length(vFolderPath));
    END LOOP;

    RETURN vFolderId;

  EXCEPTION
  WHEN no_data_found THEN
    RETURN NULL;
  END getFolderIdByPath;

  /*
    PROCEDURE:    updateScenarioXML
    DESCRIPTION:  This procedure handles adding a new instance to a User Scenario
    PARAMS:			pInstanceId - the full key of the instance to be added
        			pEntity      - the entity name of the instance
        			pScenario    - the name of the target Scenario to add to
    */
  PROCEDURE updatescenarioxml(pInstanceId IN wmn_instance.instance_id%TYPE,
                              pEntityId IN wmn_entity.entity_id%TYPE,
                              pscenarioqueryid IN wmq_query.query_id%TYPE)
  IS
    vXml CLOB;
    vfilter CLOB;
    vunlinked CLOB;

  BEGIN
  	/*
  	log_msg('');
    log_msg(' ---- Starting updateScenarioXML with the following params ----');
    log_msg('pInstanceId: ' || pInstanceId);
    log_msg('pEntityId: ' || pEntityId);
    log_msg('pScenarioQueryId: ' || pscenarioqueryid);
    log_msg('');
	*/


    /* Get the XML string relating to this entity and parse out the Filter CData section */
    vXML := getXML(pScenarioQueryId, pEntityId);

    /* Get the Filter CData from the XML string */
    vfilter := getfiltercdata(vxml);

    /* Add the new instance to the CData section if its not there already */
	vFilter := addInstancetoFilterCData(vFilter, pinstanceid);

    /* Update the XML string */
    vXML := setFilterCData(vXML, vfilter);

    /* Get the Unlinked CData from the XML string */
    vUnlinked := getunlinkedcdata(vxml);

    /* Add the new instance to the CData section it if its not there already */
	vUnlinked := addInstancetoUnlinkedCData(vUnlinked, pInstanceId);

    /* Update the XML string */
    vXML := setUnlinkedCData(vXML, vUnlinked);

    /* Now that we've fully updated the XML string we can write it back to the table */
    saveXML(vXML, pscenarioqueryid, pEntityId);

  END;

  /*
    PROCEDURE:    walkUpEntityTree
    DESCRIPTION:  This procedure gets every instance that is above this one in the instance
    			  hierarchy and it updates its XML to show that its included in the scenario.
                  It also needs to exclude sibling instances that will otherwise automatically
                  be included because their parents will now be in the scenario
    PARAMS:			pInstanceId - the full key of the instance to be added
        			pEntity      - the entity name of the instance
        			pScenario    - the id of the target Scenario to add to
    */
  PROCEDURE walkUpentitytree(pinstanceid IN wmn_instance.instance_id%TYPE,
                           pentityId IN wmn_entity.entity_id%TYPE,
                           pscenarioqueryid IN wmq_query.query_id%TYPE)
  IS

  CURSOR vupChildInstance IS
     SELECT instance_id, Parentinstance_ID, entity_id, LEVEL lvl
     FROM wmn_instance_Context
     CONNECT BY instance_id = PRIOR Parentinstance_ID
     AND Context_ID = PRIOR Context_ID
     START WITH instance_id = pinstanceid
     AND Context_ID = gContextId
     ORDER BY lvl;

  BEGIN

    log_msg('');
    log_msg('*******************************************');
    log_msg('Beginning walk up  ....');
    log_msg('pInstanceId  ....' || pInstanceId || ' -  ');
    log_msg('pentityId  ....' || pentityid || ' -  ');
    log_msg('pscenarioqueryid  ....' || pscenarioqueryid);



     -- Loop through each child
     FOR vupParentinstanceid IN vupChildInstance LOOP

		-- the first row is the current instance - this has already been updated
      if vupParentinstanceid.lvl > 0 then
        	updatescenarioxml(vupParentinstanceid.instance_id,   vupParentinstanceid.entity_id,   pscenarioqueryid);

        	/* We need to exclude the siblings which have the same parent but are of different entities */
        	excludeSiblings(vupParentinstanceid.instance_id, pentityId, pscenarioqueryid);

        END IF;

      END LOOP;

  END;


/*
    PROCEDURE:    walkDownEntityTree
    DESCRIPTION:  This procedure gets every instance that is below this one in the instance
    				hierarchy and it updates its XML to show that its included in the scenario
    PARAMS:			pInstanceId - the full key of the instance to be added
        			pScenario    - the id of the target Scenario to add to
    */
  PROCEDURE walkDownentitytree(pinstanceid IN wmn_instance.instance_id%TYPE,
                           pscenarioqueryid IN wmq_query.query_id%TYPE)
  IS

  CURSOR vdwnChildInstance IS
  SELECT instance_id, entity_id, level lvl
  FROM wmn_instance_context
  START WITH instance_id = pinstanceid
        AND context_id = gContextId
  CONNECT BY PRIOR instance_id = parentinstance_id
       AND context_id = PRIOR context_id
  ORDER BY lvl;

  BEGIN

    -- Loop through each childbut skip the first one since we've already updated it

      FOR vdwnparentinstanceid IN vdwnChildInstance LOOP

        IF vdwnparentinstanceid.lvl > 0 then
        	updatescenarioxml(vdwnparentinstanceid.instance_id,   vdwnparentinstanceid.entity_id,   pscenarioqueryid);
        END IF;

      END LOOP;

  END;


  /*
    PROCEDURE:    excludeSiblings
    DESCRIPTION:  There are two types of children in the instance tree
    					(i) instances of the same entity type
                        (ii) instances of different entity types
                  Once the parent instance has been added we need to exclude any instances of
                  different entity types - these will automatically appear once the parent has
                  been added if this is not done
    PARAMS:			pParentInstanceId - the instance Id of the Parent instance
        			pEntity      - the entity id of the current instance
        			pScenario    - the name of the target Scenario to add to
    */
  PROCEDURE excludeSiblings(pParentInstanceid IN wmn_instance.instance_id%TYPE,
                           pEntityId IN wmn_entity.entity_id%TYPE,
                           pscenarioqueryid IN wmq_query.query_id%TYPE)
  IS
    vXML	CLOB;
    vfilter	CLOB;

  	CURSOR vExcludeEntities IS
  	select 	ENTITY_ID
    from 	WMN_INSTANCE_CONTEXT
    where	PARENTINSTANCE_ID = pParentInstanceid
    and		ENTITY_ID != pEntityId
    and		CONTEXT_ID = gContextId
    GROUP BY ENTITY_ID;

  BEGIN

     -- Loop through each child
     FOR vSiblingEntityId IN vExcludeEntities LOOP

        vXML := getXML(pScenarioQueryId, vSiblingEntityId.Entity_Id);

        /* Get the Filter CData from the XML string */
        vfilter := getFilterCData(vxml);

        /* Since this entity type is different than our current one we need to exclude it */
        vfilter := excludeInstanceToFilterCData(vfilter, pParentInstanceid);

        /* Update the XML string */
	    vXML := setFilterCData(vXML, vfilter);

        /* Now that we've fully updated the XML string we can write it back to the table */
    	saveXML(vXML, pscenarioqueryid, vSiblingEntityId.Entity_Id);

     END LOOP;

  END;

  /*
    PROCEDURE:    getFilterCData
    DESCRIPTION:  This procedure takes an XML which represents one entity. It then uses
    				a DOM parser to walk the XML structure and to find the CDATA
                    element associated with the FILTER node
    PARAMS:			pXMLin - the scenario XML to be parsed
    RETURNS:		a string representing the CDATA section
    */

  FUNCTION getfiltercdata(pxmlin CLOB) RETURN CLOB
  IS
  	vxmlin 			CLOB;
    vparser 		xmlparser.parser;
    vdoc 			xmldom.domdocument;
    vfilternodes 	xmldom.domnodelist;
    vfilternode 	xmldom.domnode;
    vdatanodes 		xmldom.domnodelist;
    vdatanodecount 	NUMBER;
    vdatanode 		xmldom.domnode;
    cdatanode 		xmldom.domcharacterdata;
    vfiltercdata 	CLOB;

  BEGIN

    vxmlin := pxmlin;

    -- Create an xmlParser and use it to parse the XML string
    vparser := xmlparser.newparser();
    xmlparser.parseclob(vparser,   vxmlin);

    -- get the DOM document reference
    vdoc := xmlparser.getdocument(vparser);

    -- Get the Node that relates to the FILTER
    vfilternodes := xmldom.getelementsbytagname(vdoc,   'FILTER');

    -- Get the first of the FILTER nodes
    vfilternode := xmldom.item(vfilternodes,   0);

    -- Check whether this node has a child node
    vdatanodes := xmldom.getchildnodes(vfilternode);
    vdatanodecount := xmldom.getlength(vdatanodes);

    IF(vdatanodecount = 0) THEN
      log_msg('ERROR - No child CData Nodes found for FILTER Node');
    END IF;

    -- Get the first of the CDATA nodes and check that it is the right type
    vdatanode := xmldom.item(vdatanodes,   0);

    IF(xmldom.getnodetype(vdatanode) != xmldom.cdata_section_node) THEN
      log_msg('ERROR - Wrong type of child node found for CData');
    END IF;

    -- Have to convert the DataNote into the correct Character Data type to get the data from it
    cdatanode := xmldom.makecharacterdata(vdatanode);
    vfiltercdata := xmldom.getdata(cdatanode);

    RETURN vfiltercdata;

  END;

/*
    PROCEDURE:    setFilterCData
    DESCRIPTION:  This procedure takes in an XML string which represents one entity and also a new Filter CData string.
    				It then uses a DOM parser to walk the XML structure and to find the CDATA
                    element associated with the FILTER node and it replaces it with the new string
                    passed in. It then returns the newly constructed XML string
    PARAMS:			pXMLin - the scenario XML to be parsed
    				pCDatain - the new CData string for the FILTER node
    RETURNS:		the newly constructed XML clob
    */

  FUNCTION setfiltercdata(pxmlin CLOB, pCDataIn CLOB) RETURN CLOB
  IS
  	vxmlin 			CLOB;
    vparser 		xmlparser.parser;
    vdoc 			xmldom.domdocument;
    vfilternodes 	xmldom.domnodelist;
    vfilternodecount NUMBER;
    vfilternode 	xmldom.domnode;
    vdatanodes 		xmldom.domnodelist;
    vdatanodecount 	NUMBER;
    vdatanode 		xmldom.domnode;
    cdatanode 		xmldom.domcharacterdata;
    vfiltercdata 	CLOB;
    vXMLOut  		CLOB:=NULL;

  BEGIN

    vxmlin := pxmlin;

    -- Create an xmlParser and use it to parse the XML string
    vparser := xmlparser.newparser();
    xmlparser.parseclob(vparser,   vxmlin);

    -- get the DOM document reference
    vdoc := xmlparser.getdocument(vparser);

    -- Get the Node that relates to the FILTER
    vfilternodes := xmldom.getelementsbytagname(vdoc,   'FILTER');
    vfilternodecount := xmldom.getlength(vfilternodes);

    -- Get the first of the FILTER nodes
    vfilternode := xmldom.item(vfilternodes,   0);

    -- Check whether this node has a child node
    vdatanodes := xmldom.getchildnodes(vfilternode);
    vdatanodecount := xmldom.getlength(vdatanodes);

    IF(vdatanodecount = 0) THEN
      log_msg('ERROR - No child CData Nodes found for FILTER Node');
    END IF;

    -- Get the first of the CDATA nodes and check that it is the right type
    vdatanode := xmldom.item(vdatanodes,   0);

    IF(xmldom.getnodetype(vdatanode) != xmldom.cdata_section_node) THEN
      log_msg('ERROR - Wrong type of child node found for CData');
    END IF;

    -- Have to convert the DataNote into the correct Character Data type to get the data from it
    cdatanode := xmldom.makecharacterdata(vdatanode);
    vfiltercdata := xmldom.getdata(cdatanode);

    -- Set the CData section to be the string that was passed in
    xmldom.setData(cDataNode, pCDataIn);

    DBMS_LOB.CreateTemporary(vXMLout, TRUE);
    xmlDom.writeToClob(vDoc, vXMLout);
    RETURN vXMLout;

  END;

  /*
    PROCEDURE:    getUnlinkedCData
    DESCRIPTION:  This procedure takes an XML which represents one entity. It then uses
    				a DOM parser to walk the XML structure and to find the CDATA
                    element associated with the DataMember='Unlinked' node
    PARAMS:			pXMLin - the scenario XML to be parsed
    RETURNS:		a string representing the CDATA section
    */

    FUNCTION getunlinkedcdata(pxmlin CLOB) RETURN CLOB
    IS
	    vxmlin 			CLOB;
  		vparser 		xmlparser.parser;
  		vdoc 			xmldom.domdocument;
		vdmnodes 		xmldom.domnodelist;
  		vdmnodecount 	NUMBER;
  		vdmnode 		xmldom.domnode;
  		vdatanodes 		xmldom.domnodelist;
  		vdatanodecount 	NUMBER;
  		vdatanode 		xmldom.domnode;
  		cdatanode 		xmldom.domcharacterdata;
  		i 				INTEGER;
  		vnodemap 		xmldom.domnamednodemap;
  		vattrcount 		NUMBER;
  		vattrname 		VARCHAR2(100);
  		vattrval 		VARCHAR2(100);
  		vattrnode 		xmldom.domnode;
  		vunlinkedcdata 	CLOB;

  BEGIN

    vxmlin := pxmlin;

    -- Create an xmlParser and use it to parse the XML string
    vparser := xmlparser.newparser();
    xmlparser.parseclob(vparser,   vxmlin);

    -- get the DOM document reference
    vdoc := xmlparser.getdocument(vparser);

    -- Get the Node that relates to the DATAMEMBER
    vdmnodes := xmldom.getelementsbytagname(vdoc,   'DATAMEMBER');
    vdmnodecount := xmldom.getlength(vdmnodes);

    -- Loop through each of the DATAMEMBER nodes
    FOR i IN 0 .. vdmnodecount -1
    LOOP

      vdmnode := xmldom.item(vdmnodes,   i);

      -- get all attributes of element
      vnodemap := xmldom.getattributes(vdmnode);

      IF(xmldom.isnull(vnodemap) = TRUE) THEN
        log_msg('No attributes found for this node ');
      END IF;

      vattrcount := xmldom.getlength(vnodemap);

      -- loop through attributes
      FOR i IN 0 .. vattrcount -1
      LOOP
        vattrnode := xmldom.item(vnodemap,   i);
        vattrname := xmldom.getnodename(vattrnode);
        vattrval := xmldom.getnodevalue(vattrnode);

        -- Find the 'Unlinked attribute'

        IF(vattrval = 'Unlinked') THEN

          -- Check whether this DataMember node has a child node
          vdatanodes := xmldom.getchildnodes(vdmnode);
          vdatanodecount := xmldom.getlength(vdatanodes);

          IF(vdatanodecount = 0) THEN
            log_msg('ERROR - No child CData Nodes found for FILTER Node');
          END IF;

          -- Get the first of the CDATA nodes and check that it is the right type
          vdatanode := xmldom.item(vdatanodes,   0);

          IF(xmldom.getnodetype(vdatanode) != xmldom.cdata_section_node) THEN
            log_msg('ERROR - Wrong type of child node found for CData');
          END IF;

          -- Have to convert the DataNote into the correct Character Data type to get the data from it
          cdatanode := xmldom.makecharacterdata(vdatanode);
          vunlinkedcdata := xmldom.getdata(cdatanode);

        END IF;

      END LOOP;

    END LOOP;

    RETURN vunlinkedcdata;

  END;


/*
    PROCEDURE:    setUnlinkedCData
    DESCRIPTION:  This procedure takes an XML which represents one entity and also a new Unlinked CData string.
    				It then uses a DOM parser to walk the XML structure and to find the CDATA
                    element associated with the DataMember='Unlinked' node and updates it with the new value passed in
    PARAMS:			pXMLin - the scenario XML to be parsed
    				pCDataIn - the CData string to be inserted
    RETURNS:		a new constructed XML Clob
    */


  FUNCTION setunlinkedcdata(pxmlin CLOB, pCDataIn CLOB) RETURN CLOB
    IS
    vxmlin 		CLOB;
    vparser 	xmlparser.parser;
    vdoc 		xmldom.domdocument;
    vdmnodes 	xmldom.domnodelist;
    vdmnodecount NUMBER;
    vdmnode 	xmldom.domnode;
    vdatanodes 	xmldom.domnodelist;
    vdatanodecount NUMBER;
    vdatanode 	xmldom.domnode;
    cdatanode 	xmldom.domcharacterdata;
    i 			INTEGER;
    vnodemap 	xmldom.domnamednodemap;
    vattrcount 	NUMBER;
    vattrname 	VARCHAR2(100);
    vattrval 	VARCHAR2(100);
    vattrnode 	xmldom.domnode;
    vunlinkedcdata CLOB;
    vXMLOut 	CLOB;

  BEGIN

    vxmlin := pxmlin;

    -- Create an xmlParser and use it to parse the XML string
    vparser := xmlparser.newparser();
    xmlparser.parseclob(vparser,   vxmlin);

    -- get the DOM document reference
    vdoc := xmlparser.getdocument(vparser);

    -- Get the Node that relates to the DATAMEMBER
    vdmnodes := xmldom.getelementsbytagname(vdoc,   'DATAMEMBER');
    vdmnodecount := xmldom.getlength(vdmnodes);

    -- Loop through each of the DATAMEMBER nodes
    FOR i IN 0 .. vdmnodecount -1
    LOOP

      vdmnode := xmldom.item(vdmnodes,   i);

      -- get all attributes of element
      vnodemap := xmldom.getattributes(vdmnode);

      IF(xmldom.isnull(vnodemap) = TRUE) THEN
        log_msg('No attributes found for this node ');
      END IF;

      vattrcount := xmldom.getlength(vnodemap);

      -- loop through attributes
      FOR i IN 0 .. vattrcount -1
      LOOP
        vattrnode := xmldom.item(vnodemap,   i);
        vattrname := xmldom.getnodename(vattrnode);
        vattrval := xmldom.getnodevalue(vattrnode);

        -- Find the 'Unlinked attribute'
        IF(vattrval = 'Unlinked') THEN

          -- Check whether this DataMember node has a child node
          vdatanodes := xmldom.getchildnodes(vdmnode);
          vdatanodecount := xmldom.getlength(vdatanodes);

          IF(vdatanodecount = 0) THEN
            log_msg('ERROR - No child CData Nodes found for FILTER Node');
          END IF;

          -- Get the first of the CDATA nodes and check that it is the right type
          vdatanode := xmldom.item(vdatanodes,   0);

          IF(xmldom.getnodetype(vdatanode) != xmldom.cdata_section_node) THEN
            log_msg('ERROR - Wrong type of child node found for CData');
          END IF;

          -- Have to convert the DataNote into the correct Character Data type to get the data from it
          cdatanode := xmldom.makecharacterdata(vdatanode);
          vunlinkedcdata := xmldom.getdata(cdatanode);

          -- Set the CData section to be the string that was passed in
          xmldom.setData(cDataNode, pCDataIn);

          DBMS_LOB.CreateTemporary(vXMLout, TRUE);
          xmlDom.writeToClob(vDoc, vXMLout);

        END IF;

      END LOOP;

    END LOOP;

    RETURN vXMLout;

  END;


  /*
    PROCEDURE:    addInstancetoFilterCData
    DESCRIPTION:  This procedure takes in a Filter CData string and adds the instance Id and
    				the Parent Instance Id onto it
                    The Filter CData strings will be in one of the following formats:

                    (i)  ACTIVE
	                (ii) inGroup(Id,"7007","7008") OR (!inGroup(ParentId,"1967","1962") AND Active)
                    (iii) !inGroup(ParentId,"1963") AND Active

                    In the first case there are no instances of this entity in the scenario yet
                    In the second case the first part is the group of instances that are included in this scenario
                    	The second part is the group of their parent instances
                    	When adding a new instance to the scenario we will append its details to both
                    In the third case this is an excluded subling entity - we'll need to change this into the format (ii)

    PARAMS:			pInstanceId - the id of the instance to be added

    */
  FUNCTION addInstancetoFilterCData(pCData CLOB,
  									pinstanceid IN wmn_instance.instance_id%TYPE) RETURN CLOB
  IS
	vCData				CLOB;
    vParentInstanceId	wmn_instance.instance_id%TYPE;
    vInstList			CLOB;
    vParentInstList 	CLOB;
    vListEnd 			INTEGER;
    vListStart 			INTEGER;

  BEGIN

    --log_msg(pinstanceid || '  -  ' || pCData);

    -- Check that the instance isn't already in the string
    if (INSTR( pCData, '"' || pinstanceid || '"' ) > 0 ) then

    	--log_msg('The instance ' || pInstanceId || 'is already in the string');
		vCData := pcData;

    else

        /* Get the parent instance id as we'll need to add it to the CData string */
        select 	PARENTINSTANCE_ID
        into vParentInstanceId
        from	WMN_INSTANCE_CONTEXT
        where	INSTANCE_ID = pInstanceId
        and		CONTEXT_ID = gContextId;

        /* Case (i) */
        if (upper(pCData) = 'ACTIVE') then

            /* This is the most basic version of the string with no instances in it - we need to
            	add the instance in the new format */
    		vInstList := 'inGroup(Id, ' || '"' || pInstanceId ||'")';
            vParentInstList := ' OR (!inGroup(ParentId, ' || '"' || vParentInstanceId || '"' || ') AND Active)';

        /* Case (ii) */
        elsif (substr(pcData, 1, 7) = 'inGroup' ) then
        	/* The CData section already has some instances in it - we can add to these */
            /* Parse the string to get the list of instances */
            vListStart := INSTR( pCData, 'inGroup(Id,');
            if (vListStart > 0) then
                vListEnd := INSTR( pCData, ')', vListStart);
                vInstList := TRIM(SUBSTR(pCData, vListStart + 11, vListEnd - (vListStart + 11)));

                /* Add the new instance onto the instance list */
            	vInstList := vInstList || ',"' || pInstanceId || '"';
            	vInstList := 'inGroup(Id, ' || vInstList ||')';

    	    end if;

            /* Now parse the string to get the list of Parent instances */
            vListStart := INSTR( pCData, 'inGroup(ParentId,');
            if (vListStart > 0) then
                vListEnd := INSTR( pCData, ')', vListStart);
                vParentInstList := SUBSTR(pCData, vListStart + 17, vListEnd - (vListStart + 17));

            --    dbms_output.put_line(pinstanceid || ' - PARENT data before - ' || vParentInstList);

                /* Add its parent instance onto the Parent Instance list if its not there already */
                if (INSTR( vParentInstList, '"' || vParentInstanceId || '"' ) = 0 ) then
	            	vParentInstList := vParentInstList || ',"' || vParentInstanceId || '"';
                end if;

            	vParentInstList := ' OR (!inGroup(ParentId, ' || vParentInstList || ') AND Active)';

			--	dbms_output.put_line(pinstanceid || ' - PARENT data after - ' || vParentInstList);

    	    end if;

        /* Case (iii) */
        elsif ((substr(pcData, 1, 8) = '!inGroup' ) OR (substr(pcData, 1, 9) = '(!inGroup' ))then

           -- log_msg('Found a excluded sibling');

			/* The filter was set to exclude but now we can reformat it to include this instance */
    		vInstList := 'inGroup(Id, ' || '"' || pInstanceId ||'")';
            vParentInstList := ' OR (!inGroup(ParentId, ' || '"' || vParentInstanceId || '"' || ') AND Active)';

        end if;

        /* Now can build the new CData string from the two lists */
        vCData := vInstList || vParentInstList;

    end if;

    RETURN vCData;

  END;


  /*
    PROCEDURE:    excludeInstancetoFilterCData
    DESCRIPTION:  This procedure takes in a Filter CData string excludes all of its children from
    				the scenario - this is necessary for siblings that are of different entity type

    PARAMS:			pcData - the existing Filter CData section
    				pInstanceId - the id of the instance to be added

    */
  FUNCTION excludeInstancetoFilterCData(pCData CLOB,
	pParentinstanceid IN wmn_instance.instance_id%TYPE) RETURN CLOB
  IS
  vCData			CLOB;
    vParentInstList CLOB;

  BEGIN

    -- Check that the instance isn't already in the string
    if (INSTR( pCData, '"' || pParentinstanceid || '"' ) > 0 ) then

    	--log_msg('The Parent instance ' || pParentinstanceid || 'is already in the string');
		vCData := pcData;

    /* We only make a change to this if its set to ACTIVE */
	elsif (upper(pCData) = 'ACTIVE') then

      	/* This is the most basic version of the string with no instances in it - we need to
	  	add the instance in the new format */
      	vParentInstList := '(!inGroup(ParentId, ' || '"' || pParentinstanceid || '"' || ') AND Active)';

	  	/* Now can build the new CData string from the two lists */
      	vCData := vParentInstList;

	else
    	/* No change to the CData section */
		vCData := pcData;
	end if;

    RETURN vCData;

  END;

  /*
    PROCEDURE:    addInstancetoUnlinkedCData
    DESCRIPTION:  This procedure takes in an Unlinked CData string and adds the instance Id to it
                    The Unlinked CData strings will be in the following format:

                    (i)  !ACTIVE
	                (ii) inGroup(Id,"18221","18222") OR !Active

                    In the first case there are no instances of this entity in the scenario yet
                    In the second case the first part is the group of instances that are included in this scenario
                    When adding a new instance to the scenario we will append its details to both

    PARAMS:			pInstanceId - the id of the instance to be added

    */
  FUNCTION addInstancetoUnlinkedCData(pCData CLOB,
  									  pinstanceid IN wmn_instance.instance_id%TYPE) RETURN CLOB
  IS
	vCData		CLOB;
    vInstList	CLOB;
    vListEnd 	INTEGER;
    vListStart 	INTEGER;

  BEGIN

    -- Check that the instance isn't already in the string
    if (INSTR( pCData, '"' || pinstanceid || '"' ) > 0 ) then

    	--log_msg('The instance ' || pInstanceId || ' is already in the CData string');
        vCData := pcData;

    else

        if (upper(pCData) = '!ACTIVE') then

            /* This is the most basic version of the string with no instances in it - we need to
            	add the instance in the new format */
    		vInstList := '"' || pInstanceId ||'"';

        else
        	/* The CData section already has some instances in it - we can add to these */
            /* Parse the string to get the list of instances */
            vListStart := INSTR( pCData, 'inGroup(Id,');
            if (vListStart > 0) then
                vListEnd := INSTR( pCData, ')', vListStart);
                vInstList := TRIM(SUBSTR(pCData, vListStart + 11, vListEnd - (vListStart + 11)));
    	    end if;

			-- dbms_output.put_line(pinstanceid || ' - UNLINKED data before - ' || vInstList);


            /* Add the new instance onto the instance list */
            vInstList := vInstList || ',"' || pInstanceId || '"';

			-- dbms_output.put_line(pinstanceid || ' - UNLINKED data after - ' || vInstList);

        end if;

        /* Now can build the new CData string from the two lists */
        vCData := 'inGroup(Id, ' || vInstList ||') OR !Active';

    end if;

    RETURN vCData;

  END;


  /*
    FUNCTION:     getXML
    DESCRIPTION:  This function gets the XML statement that makes up the instances that are to be
    				included for this entity. This statement should normally be on a single row in the
                    wmq_query_text table but if the XML statment > 4000 chars then it will be rolled onto
                    subsequent lines

    PARAMS:			pScenarioQueryId - the id of the scenario to be updated
                    pEntityId - the id of the entity
    */
  FUNCTION getXML(pScenarioQueryId IN wmq_query.query_id%TYPE,
                    pEntityId	WMN_ENTITY.ENTITY_ID%TYPE) RETURN CLOB
  IS

  	vRow		WMQ_QUERY_TEXT.QUERY_ROW%TYPE;
    vEntityName WMN_ENTITY.ENTITY_NAME%TYPE;
    vXML		CLOB;
    vNextRow	WMQ_QUERY_TEXT.QUERY_TEXT%TYPE;
    vFound		INTEGER := NULL;

  BEGIN


    /* Get the entity name to use in getting XML */
	SELECT ENTITY_NAME
    into vEntityName
    FROM WMN_ENTITY
    WHERE ENTITY_ID = pEntityId;


    /* Get the XML string relating to this entity and parse out the Filter CData section */

    SELECT query_text, query_row
    INTO vXml, vRow
    FROM wmq_query_text
    WHERE query_id = pScenarioQueryId
      AND (query_text LIKE '<CLASS Name=''' || vEntityName || '''%'
      OR  query_text LIKE '<CLASS Name="' || vEntityName || '"%');


    /* Check the next row to see if its a continuation */
    WHILE vFound is null
    LOOP
    	vRow := vRow + 1;

        SELECT query_text
        INTO vNextRow
        FROM wmq_query_text
        WHERE query_id = pScenarioQueryId
        AND query_row = vRow;

		/* If it doesn't contain a class def then its part of the previous entities XML string */
        if (SUBSTR(vNextRow, 1, 6) != '<CLASS') then
        	vXML := vXML || vNextRow;
    	else
        	--log_msg('Finished peek - found next new entity at : ' || vRow);
        	vFound := 1;
        end if;
    END LOOP;

    RETURN vXML;

  END;


  /*
    PROCEDURE:    saveXML
    DESCRIPTION:  This procedure takes in the XML string and saves it back to the WMQ_QUERY_TEXT
    				table. Since the max length of the column in the table is 4000 then we need to
                    chop the XML string up to save it back

    PARAMS:			pXML - the XML string to be saved,
    				pScenarioQueryId - the id of the scenario to be updated
                    pEntityId - the id of the entity
    */
  PROCEDURE saveXML(pXML CLOB,
  					pscenarioqueryid IN wmq_query.query_id%TYPE,
                    pEntityId	WMN_ENTITY.ENTITY_ID%TYPE)
  IS

    vEntityName 	WMN_ENTITY.ENTITY_NAME%TYPE;
	vXmlScrub 		CLOB;
    vXMLOut			VARCHAR2(4000);
    vChopAmount 	INTEGER := 4000;
    vStartChop		INTEGER := 1;
    vRowOut			WMQ_QUERY_TEXT.QUERY_ROW%TYPE;
    vPeek			INTEGER;

  BEGIN


    /* Get the entity name to use in saving XML */
	SELECT ENTITY_NAME
    into vEntityName
    FROM WMN_ENTITY
    WHERE ENTITY_ID = pEntityId;


	/* Get the correct position to start writing out at */
	SELECT query_row
    INTO vRowOut
    FROM wmq_query_text
    WHERE query_id = pScenarioQueryId
     AND (query_text LIKE '<CLASS Name=''' || vEntityName || '''%'
     OR  query_text LIKE '<CLASS Name="' || vEntityName || '"%');


	/* Scrub the XML string to get it back to the same state as the other ones */
	vXmlScrub := REPLACE(pXML, chr(10), '');
	vXmlScrub := REPLACE(vXmlScrub, '>  <', '> <');


    /* The query text column is only 4000 chars long so if the XML string is longer than that
    then we need to chop it up and save it on subsequent rows */

    WHILE (vStartChop < LENGTH(vXMLScrub))
    LOOP

    	vXmlout := substr(vXmlScrub, vStartChop, vChopAmount);

		/* Write the chopped string out to the correct row */

        if vStartChop > 1 then
    		/* Once the first row has been updated things get a bit tricky - if the following row
            is the row for another entity then we need to create space by forcing the rest up -
            otherwise just update as normal */
            SELECT count(*)
            INTO vPeek
            FROM wmq_query_text
            WHERE query_id = pScenarioQueryId
             AND query_text LIKE '<CLASS Name=%'
             AND query_row = vRowOut;

            if (vPeek > 0) then

                /* Move all the other rows down by 1 to make room for the new row */
                UPDATE wmq_query_text
                SET query_row = query_row + 1
                WHERE query_id = pScenarioQueryId
                AND query_row >= vRowOut;

                /* Insert a new row for this part of the XML */
                INSERT INTO wmq_query_text(query_id, query_row, query_text)
                values (pScenarioQueryId, vRowOut, vXmlout);

			else
            	/* Update as per normal */
            	UPDATE wmq_query_text
          	    SET		query_text = vXmlout
          	    WHERE 	query_id = pscenarioqueryid
          	    AND 	query_row = vRowOut;

            end if;

        else
        	/* This is the first row so its safe to update directly */
    		UPDATE wmq_query_text
    	    SET		query_text = vXmlout
    	    WHERE 	query_id = pscenarioqueryid
    	    AND 	query_row = vRowOut;
	    end if;

        /* Increment counters */
		vStartChop := vStartChop + vChopAmount;
        vRowOut := vRowOut + 1;

    END LOOP;


  END;


  /*
    PROCEDURE:    backupScenario
    DESCRIPTION:  This procedure writes the contents of a scenario out to a file

    PARAMS:			pscenario - the scenario to be exported
					retcode - the return code
    */
  PROCEDURE backupScenario(pFolderpath IN VARCHAR2,
                           pscenario IN PM_SCENARIO.NAME%TYPE,
                           retcode   OUT     NUMBER)
  IS
   -- File declarations
   rpt_file            utl_file.file_type;

   -- All local variables (not tied to a database field)
   rpt_line                  VARCHAR2(5000);

   vfilename                 varchar(70);

   vcnt                     number := 0;

   vFolderId	pm_folder.id%TYPE;
   vscenarioqueryid pm_scenario.query_id%TYPE;

   invalid_folder EXCEPTION;
   invalid_scenario EXCEPTION;

  -- Cursor to find all

  CURSOR scenario IS

       SELECT query_id, query_row, query_text
       FROM wmq_query_text
       WHERE query_id = vscenarioqueryid;

   /*========================================================================*/
   /* WRITE QUERY TEXT TO FILE
   /*========================================================================*/

  BEGIN
    retcode := 0;

    vfilename := pscenario || '_' || to_char(sysdate, 'YYYYMMDDHH24MISS') || '.bkp';

    BEGIN
      rpt_file  := utl_file.fopen('/tmp', vfilename, 'W',32767);
    EXCEPTION
       WHEN OTHERS THEN
          dbms_output.put_line('Error opening file ' ||TO_CHAR(SQLCODE, '99999'));
          retcode := 1;
          RETURN;
    END;

    /* Get the folder id */
    vFolderId := getFolderIdByPath(pFolderpath);

    IF vFolderId IS NULL THEN
      raise invalid_folder;
    END IF;

    /* Get the scenario Id */
    vscenarioqueryid := validatescenario(pscenario, vFolderId);

    IF vscenarioqueryid IS NULL THEN
      raise invalid_scenario;
    END IF;

    FOR scenario_recd IN scenario LOOP
      --dbms_output.put_line(scenario_recd.query_id || ',' ||  scenario_recd.query_row || ',' || scenario_recd.query_text);
      rpt_line :=  scenario_recd.query_id || ',' ||  scenario_recd.query_row || ',' || scenario_recd.query_text;
      utl_file.put_line(rpt_file, rpt_line);

      vcnt := vcnt+1;

    END LOOP;

    IF vcnt = 0 THEN
      dbms_output.put_line('No records existing for this Scenario ' || pscenario);
      retcode := 1;
    ELSE
      dbms_output.put_line('Backup File ' || vfilename || ' created for Scenario ' || pscenario);
      retcode := 0;
    END IF;

    utl_file.fclose(rpt_file);

  EXCEPTION
       WHEN invalid_folder THEN
	log_msg('======================================================= ');
	log_msg('Unable to resolve folder path ' || pFolderPath );
	log_msg('======================================================= ');
	retcode := 1;
        WHEN invalid_scenario THEN
	log_msg('======================================================= ');
	log_msg('Scenario' || vscenarioqueryid || ' is not valid');
	log_msg('======================================================= ');
	retcode := 1;
       WHEN OTHERS THEN
       dbms_output.put_line('Error writing Backup File ' || sqlcode );
       retcode := 1;

  END backupScenario;


  /*
    PROCEDURE:    restoreScenario
    DESCRIPTION:  This procedure loads a scenario from a file

    PARAMS:			pFile - the scenario to be exported
					retcode - the return code
    */
  PROCEDURE restoreScenario(pfile IN PM_SCENARIO.NAME%TYPE,
                          retcode   OUT     NUMBER)
  IS

    dataline                VARCHAR2(5000);

    in_file                 utl_file.file_type;

    vquery_id		wmq_query_text.query_id%type;
    vquery_row		wmq_query_text.query_row%type;
    vquery_text		wmq_query_text.query_text%type;

    vpos1			number := 0;
    vpos2			number := 0;
    vlen                        number := 0;
    vcnt                        number := 0;
    vdeleted                    number := 0;

  BEGIN

    retcode := 0;

    BEGIN
       in_file := UTL_FILE.FOPEN('/tmp', pfile, 'R',32767);
    EXCEPTION
      WHEN OTHERS THEN
      dbms_output.put_line(' Error opening file ' || pfile || '.  Verify file exists in /tmp directory. ');
      retcode := 1;
      RETURN;
    END;

    LOOP
      BEGIN
      utl_file.get_line(in_file, dataline);
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
              --dbms_output.put_line('sqlcode ' ||  SQLCODE);
              EXIT;
         WHEN OTHERS THEN
             -- dbms_output.put_line('sqlcode ' ||  SQLCODE);
              retcode := 1;
      END;

      vpos1 := instr(dataline,',',1,1);

      vpos2 := instr(dataline,',',1,2);

      vlen := (vpos2 - (vpos1+1));

      vquery_id   :=  substr(dataline, 1, vpos1-1);

      vquery_row  :=  substr(dataline, (vpos1+1), vlen);

      vquery_text :=  substr(dataline, vpos2+1);

      IF vdeleted = 0 THEN
         DELETE FROM WMQ_QUERY_TEXT
         WHERE query_id = vquery_id;
         vdeleted := 1;
      END IF;

      INSERT INTO WMQ_QUERY_TEXT
      VALUES (vquery_id,vquery_row,vquery_text);

    END LOOP;

    COMMIT;
    UTL_FILE.FCLOSE(in_file);

   EXCEPTION
      WHEN OTHERS THEN
         dbms_output.put_line('sqlcode ' ||  SQLCODE);
         retcode := 1;
   END restoreScenario;



   /*
    PROCEDURE:    log_Msg
    DESCRIPTION:  This procedure logs messages to a file

    PARAMS:			p_msg_txt - the message text to be logged
    */
   PROCEDURE log_msg        (p_msg_txt     IN VARCHAR2 DEFAULT NULL)
   IS
	/*
   BEGIN
      rpt_line2 := p_msg_txt;
      utl_file.put_line(rpt_file2, rpt_line2);
   EXCEPTION
      WHEN OTHERS THEN
        dbms_output.put_line('Error Writing log file ');
   END log_msg;
    */

    BEGIN
    	dbms_output.put_line(p_msg_txt);

    END;


END WM_SCENARIOUPDATE;
/
SHOW ERRORS